#pragma once

#include <unordered_map>
#include <functional>
#include <string>
#include <list>

struct EventHandler;
struct EventTicket;
class MyPropertySystem;

using EntryName = std::string;  //to remove event
using EventId = std::string;
using EventFunc = std::function<bool(const MyPropertySystem*)>;
using EventList = std::list<EventHandler>;
using EventMap = std::unordered_map<EventId, EventList>;
using EnteryMap = std::unordered_map<EntryName, EventId>;
using EventTicketList = std::list<EventTicket>;

struct EventHandler
{
    EventId eventId;
    EventFunc func;
    EntryName entryName;
};

struct EventTicket
{
    EventId eventId;
    MyPropertySystem* pProperty;
    float durationTime;
    float startTime;
};


class EventManager
{
private:
    EventMap m_eventMap;
    EnteryMap m_entryMap;
    EventTicketList m_ticketList;

    float m_remainedTime;
    float m_accumulatedTime;

public:
    EventManager();
    ~EventManager();
    bool AddEvent(EventId eventId, EventFunc eventFunc);
    bool AddEvent(EventId eventId, EventFunc eventFunc, EntryName entryName);
    bool RemoveEvent(EntryName entryName);
    void SendEvent(EventId eventId, const MyPropertySystem& param);
    void PostEvent(EventId eventId, const MyPropertySystem& param, float duration); //duration in seconds
    void Update(float deltaTime);
    float GetAccumuatedTime() { return m_accumulatedTime; }
};

