#include <stdio.h>
# include <conio.h>  
#include <string>
#include <iostream>
#include "EventManager.h"
#include "property/PropertyType.h"
#include "property/MyProperty.h"
#include "property/MyPropertySystem.h"

#include <chrono>
#include <Windows.h>

struct Car
{
    std::string name;
    int ShowPrice(const MyPropertySystem* propertySystem)
    {
        PropertyList list = propertySystem->GetList();
        for (auto p : list)
        {
            std::cout << name << " : " <<  p->GetName() << " : " << p->GetTypeName() << " : " << ToString(p) << std::endl;
        }
        return 0;
    }

    int ShowLowPrice(const MyPropertySystem* propertySystem)
    {
        PropertyList list = propertySystem->GetList();
        for (auto p : list)
        {
            std::cout << name << " : "  << p->GetName() << " : " << p->GetTypeName() << " : " << ToString(p) << std::endl;
        }
        return 0;
    }
};

using  namespace  std::placeholders;
void DoTest()
{
    std::cout << "============ MyProperty Test ============" << std::endl;
    MyProperty<bool> boolTest("boolTest");
    boolTest = true;
    std::cout << boolTest.GetName() << " : " << boolTest.GetTypeName() << " : " << boolTest << std::endl;
    MyProperty<int> intTest("intTest");
    intTest = 10;
    std::cout << intTest.GetName() << " : " << intTest.GetTypeName() << " : " << intTest << std::endl;
    MyProperty<std::string> stringTest("stringTest");
    stringTest = "hello world!!!";
    std::cout << intTest.GetName() << " : " << stringTest.GetTypeName() << " : " << stringTest << std::endl;

    std::cout << "============ MyPropertySystem Test ============" << std::endl;
    MyPropertySystem prop;
    prop.GetValue<int>("Damage") = 100;
    prop.GetValue<float>("hp") = 1000;
    prop.GetValue<std::string>("class") = "worrior";
    prop.GetValue<bool>("alive") = true;

    IProperty* damageProp = prop.GetProperty("Damage");
    IProperty* hpProp = prop.GetProperty("hp");
    IProperty* classProp = prop.GetProperty("class");
    IProperty* aliveProp = prop.GetProperty("alive");

    std::cout << damageProp->GetName() << " : " << damageProp->GetTypeName() << " : " << ToString(damageProp) << std::endl;
    std::cout << hpProp->GetName() << " : " << hpProp->GetTypeName() << " : " << ToString(hpProp) << std::endl;
    std::cout << classProp->GetName() << " : " << classProp->GetTypeName() << " : " << ToString(classProp) << std::endl;
    std::cout << aliveProp->GetName() << " : " << aliveProp->GetTypeName() << " : " << ToString(aliveProp) << std::endl;

    std::cout << "============ Test AddEvent: add NormalPrice, LowPrice ============" << std::endl;
    EventManager manager;
    Car car1;
    car1.name = "an electric car";
    Car car2;
    car2.name= "a gasoline car";

    manager.AddEvent("Update_Price", std::bind(&Car::ShowPrice, &car1, _1), "NormalPrice");
    manager.AddEvent("Update_Price", std::bind(&Car::ShowLowPrice, &car2, _1), "LowPrice");

    MyPropertySystem param;
    param.GetValue<int>("price") = 1000;
    param.GetValue<bool>("isAll") = true;
    manager.SendEvent("Update_Price", param);

    std::cout << "============ Test RemoveEvent : remove NormalPrice ============" << std::endl;
    manager.RemoveEvent("NormalPrice");

    MyPropertySystem param1;
    param1.GetValue<int>("price") = 100;
    param1.GetValue<float>("percent") = 50.5f;

    manager.SendEvent("Update_Price", param1);
}

void DoPostEventTest()
{
    int count = 0;

    std::cout << "============ Delay Time ============" << std::endl;
    std::chrono::high_resolution_clock::time_point old = std::chrono::high_resolution_clock::now();
    EventManager manager;

    Car car1;
    car1.name = "AAA car";
    Car car2;
    car2.name = "BBB car";

    manager.AddEvent("ID_AAA", std::bind(&Car::ShowPrice, &car1, _1), "runAAA");
    manager.AddEvent("ID_BBB", std::bind(&Car::ShowLowPrice, &car2, _1), "runBBB");

    MyPropertySystem param;
    param.GetValue<int>("price") = 1000;
    manager.PostEvent("ID_AAA", param, 3);

    MyPropertySystem param1;
    param1.GetValue<std::string>("comment") = "Best Car";
    manager.PostEvent("ID_BBB", param1, 5);

    manager.AddEvent("ID_CCC", std::bind(&Car::ShowLowPrice, &car2, _1), "runBBB");
    manager.PostEvent("ID_CCC", param1, 7);

    for (;;)
    {
        if (GetAsyncKeyState(VK_SPACE) & 0x8000 || GetAsyncKeyState(VK_ESCAPE) & 0x8000)
            break;

        std::chrono::high_resolution_clock::time_point curr = std::chrono::high_resolution_clock::now();
        std::chrono::duration<double> sec = curr - old;
        manager.Update((float)sec.count());
        std::cout << manager.GetAccumuatedTime() << "\r";
        //std::cout << count << "\r";
        old = curr;
        count++;
    }
}

int main()
{
    _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
    {
        DoTest();
        DoPostEventTest();
    }

    return 0;
}